"""
Copyright 2022 GamingCoookie

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import requests as r
import json
import datetime


class Fahrplan:
    def __init__(self, auth: str):
        self.__token = auth
        self.__request_headers = {'Accept': 'application/json',
                                  'Authorization': 'Bearer {auth}'.format(auth=self.__token)}

    def get_station_by_name(self, name: str, select_first: bool = True) -> list:
        stations = json.loads(r.get(url='https://api.deutschebahn.com/fahrplan-plus/v1/location/{x}'.format(x=name),
                                    headers=self.__request_headers).content)
        for index in range(len(stations)):
            del stations[index]['lon']
            del stations[index]['lat']

        return stations[0] if select_first else stations

    def get_arrival_board(self, board_id: int, date: datetime.datetime or datetime.date) -> list:
        if isinstance(date, datetime.datetime):
            url = 'https://api.deutschebahn.com/fahrplan-plus/v1/arrivalBoard/{bid}?date={date}'.format(
                bid=board_id, date=date.isoformat())
            url.replace(':', '%3A')
        else:
            url = 'https://api.deutschebahn.com/fahrplan-plus/v1/arrivalBoard/{bid}?date={date}'.format(
                bid=board_id, date=date.isoformat())

        board = json.loads(r.get(url=url, headers=self.__request_headers).content)

        return board

    def get_departure_board(self, board_id: int, date: datetime.datetime or datetime.date) -> list:
        if isinstance(date, datetime.datetime):
            url = 'https://api.deutschebahn.com/fahrplan-plus/v1/departureBoard/{bid}?date={date}'.format(
                bid=board_id, date=date.isoformat())
            url.replace(':', '%3A')
        else:
            url = 'https://api.deutschebahn.com/fahrplan-plus/v1/departureBoard/{bid}?date={date}'.format(
                bid=board_id, date=date.isoformat())

        board = json.loads(r.get(url=url, headers=self.__request_headers).content)

        return board

    def get_journey_details(self, journey_id: str) -> list:
        journey_id = journey_id.replace('%', '%25')
        url = 'https://api.deutschebahn.com/fahrplan-plus/v1/journeyDetails/{id}'.format(id=journey_id)

        journey = json.loads(r.get(url=url, headers=self.__request_headers).content)

        return journey



"""
Copyright 2023 Julian Metzler

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import errno
import os
import signal

from functools import wraps


def timeout(seconds, error_message = os.strerror(errno.ETIME)):
    def decorator(func):
        def _handle_timeout(signum, frame):
            raise TimeoutError(error_message)

        def wrapper(*args, **kwargs):
            signal.signal(signal.SIGALRM, _handle_timeout)
            signal.alarm(seconds)
            try:
                result = func(*args, **kwargs)
            finally:
                signal.alarm(0)
            return result

        return wraps(func)(wrapper)

    return decorator


def route_from(route, station):
    start_found = False
    new_route = []
    for stop in route:
        if not start_found:
            start_found = stop['name'].startswith(station)
        if start_found:
            new_route.append(stop)
    return new_route


def route_remove_cancelled(route):
    # Filter all cancelled stops from a route
    return [stop for stop in route if not stop.get('isCancelled')]


def get_display_delay(delay, bins):
    """
    Calculate displayed delay by rounding into provided bins.
    Example bins:
    [
        # Format: (from, to, display)
        # -1 as an upper bound means
        # "perform only lower bounds check"
        ( 0,  4,  0), # round delays from 0 to 4 minutes to 0
        ( 5,  9,  5), # round delays from 5 to 9 minutes to 5
        (10, 14, 10), # etc.
        (15, 19, 15),
        (20, 24, 20),
        (25, 29, 25),
        (30, 34, 30),
        (35, 39, 35),
        (40, 44, 40),
        (45, 49, 45),
        (50, 59, 50),
        (60, 69, 60),
        (70, 79, 70),
        (80, 89, 80),
        (90, 99, 90),
        (100, -1, -1)
    ]
    """
    for _bin in bins:
        if _bin[1] == -1:
            if delay >= _bin[0]:
                return _bin[2]
        else:
            if delay >= _bin[0] and delay <= _bin[1]:
                return _bin[2]
    return delay


"""
Copyright 2020-2024 Julian Metzler

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import datetime
import pytz
import requests


class DBInfoscreen:
    def __init__(self, host):
        self.host = host

    def get_trains(self, station):
        resp = requests.get("https://{host}/{station}.json".format(host=self.host, station=station))
        data = resp.json()
        if 'error' in data:
            return []
        return data['departures'] or []

    def get_coach_order(self, train, departure, ibnr):
        departure = departure.astimezone(pytz.utc)
        train_type = train.split()[0]
        train_number = train.split()[-1]
        if isinstance(departure, datetime.datetime):
            dep_str = departure.strftime("%Y%m%d%H%M")
        else:
            dep_str = departure
        params = {
            "administrationId": 80,
            "category": train_type,
            "date": departure.strftime("%Y-%m-%d"),
            "evaNumber": ibnr,
            "number": train_number,
            "time": departure.isoformat().replace("+00:00", "Z")
        }
        resp = requests.get("https://www.bahn.de/web/api/reisebegleitung/wagenreihung/vehicle-sequence", params=params)
        data = resp.json()
        if 'error' in data:
            return None
        return data

    def calc_real_times(self, trains, round_delay=False):
        output = []
        for train in trains:
            if train['scheduledDeparture']:
                if round_delay:
                    delay_dep = self.round_delay(train['delayDeparture'], indicate_unspecified=False)
                else:
                    delay_dep = train['delayDeparture']
                departure = datetime.datetime.strptime(train['scheduledDeparture'], "%H:%M")
                departure += datetime.timedelta(minutes=delay_dep)
                train['actualDeparture'] = departure.strftime("%H:%M")
            else:
                train['actualDeparture'] = None
            
            if train['scheduledArrival']:
                if round_delay:
                    delay_arr = self.round_delay(train['delayArrival'], indicate_unspecified=False)
                else:
                    delay_arr = train['delayArrival']
                departure = datetime.datetime.strptime(train['scheduledArrival'], "%H:%M")
                departure += datetime.timedelta(minutes=delay_arr)
                train['actualArrival'] = departure.strftime("%H:%M")
            else:
                train['actualArrival'] = None
            
            output.append(train)
        return output
    
    @staticmethod
    def round_delay(delay, indicate_unspecified=True):
        if delay <= 0:
            return 0
        
        if delay > 210 and indicate_unspecified:
            return -1 # delayed for unspecified amount of time
        
        delay_groups = (list(range(0, 60, 5)) + list(range(60, 1440, 10)))[::-1]
        for g in delay_groups:
            if delay >= g:
                return g

    @staticmethod
    def time_sort(train):
        now = datetime.datetime.now()
        now_date = now.date()
        now_time = now.time()
        if train['actualDeparture']:
            sort_time = datetime.datetime.strptime(train['actualDeparture'], "%H:%M").time()
        elif train['actualArrival']:
            sort_time = datetime.datetime.strptime(train['actualArrival'], "%H:%M").time()
        else:
            raise ValueError("Train to sort has neither actualDeparture nor actualArrival")

        result = datetime.datetime.combine(now_date, sort_time)
        if now_time >= datetime.time(12, 0) and sort_time < datetime.time(12, 0):
            result += datetime.timedelta(days=1)
        return result



name = "deutschebahn"
version = "1.4.0"
description = """Collection of utilities related to German train operator Deutsche Bahn.
Includes bindings for https://github.com/derf/db-fakedisplay, coach order API, list of stations, and more.
Includes unmodified DS 100 station data, licensed under CC BY 4.0 (https://creativecommons.org/licenses/by/4.0/deed), created by Deutsche Bahn AG."""
license = "GPLv3"
author = "Julian Metzler"
author_email = "git@mezgr.de"
requires = ['requests']
url = "https://github.com/Mezgrman/pyDeutscheBahn"
keywords = "train web api json departure arrival times coach order station list db deutsche bahn"


"""
Copyright 2020-2022 Julian Metzler

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import csv
import os


class DS100:
    def __init__(self, filename = None):
        self.ds100 = {}
        filename = filename or os.path.join(os.path.dirname(__file__), "ds100.csv")
        with open(filename, encoding='utf-8') as f:
            # Original header:
            # Abk;Name;Kurzname;Typ;Betr-Zust;Primary location code;UIC;RB;gültig von;gültig bis;Netz-Key;Fpl-rel;Fpl-Gr
            field_names = [
                'code',
                'name',
                'short_name',
                'type',
                'op_status',
                'pri_loc_code',
                'rics_code',
                'regional_area',
                'valid_from',
                'valid_till',
                'unique_key',
                'schedule_relevance',
                'schedule_edit_boundary'
            ]
            f.readline() # Skip header row since we manually define the header
            reader = csv.DictReader(f, fieldnames=field_names, delimiter=";")
            for row in reader:
                self.ds100[row['code']] = dict(row)
    
    def get(self, code):
        return self.ds100.get(code)
    
    def search_name(self, search_str):
        results = []
        for code, data in self.ds100.items():
            if search_str.lower() in data['name'].lower():
                results.append(data.copy())
        return results
    
    def filter(self, func):
        return {k: v for k, v in self.ds100.items() if func(v)}
    
    def all(self):
        return self.ds100



from setuptools import setup, find_packages

metadata = {}
with open('deutschebahn/metadata.py') as f:
    exec(f.read(), metadata)

setup(
    name = metadata['name'],
    version = metadata['version'],
    description = metadata['description'],
    license = metadata['license'],
    author = metadata['author'],
    author_email = metadata['author_email'],
    install_requires = metadata['requires'],
    url = metadata['url'],
    keywords = metadata['keywords'],
    packages = find_packages(),
    package_data = {'': ["ds100.csv"]}
)
